# Copyright 2012 Rackspace
#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import logging
from os.path import join as pjoin
from os.path import basename, splitext
from collections import defaultdict

from cliff.commandmanager import EntryPointWrapper

from cliff_rackspace.commands.help import HelpCommand

LOG = logging.getLogger(__name__)


class CommandManager(object):
    """
    Custom command manager which supports commands in the following format:
    <app> <command> <sub command>

    For example: registry services list
    """
    def __init__(self, namespace, commands_path):
        self.commands = defaultdict(dict)
        self.namespace = namespace
        self.commands_path = commands_path
        self._load_commands()

    def _load_commands(self):
        for commands_directory in os.listdir(self.commands_path):
            directory = pjoin(self.commands_path, commands_directory)

            if not os.path.isdir(directory):
                continue

            for command_file in os.listdir(directory):
                if not self._is_command_field(command_file):
                    continue

                name = splitext(basename(command_file))[0]
                module_name = os.path.dirname(__file__).split(os.sep)[-1]
                module_path = '%s.commands.%s.%s' % (self.namespace,
                                                     commands_directory,
                                                     name)
                command_class = '%sCommand' % (name.title().replace('-', ''))

                LOG.debug('Found command %s %s', directory, name)

                try:
                    mod = __import__(module_path, fromlist=[command_class])
                except ImportError, e:
                    LOG.debug('Failed to load module: %s' % (str(e)))
                    continue

                command_module = getattr(mod, command_class, None)

                if command_module is None:
                    LOG.debug('Module %s doesn\'t export %s class' %
                              (module_path, command_class))
                    continue

                wrapper = EntryPointWrapper(name=name,
                                            command_class=command_module)
                self.commands[commands_directory][name] = wrapper

    def _is_command_field(self, name):
        return name.endswith('.py') and name != '__init__.py'

    def __iter__(self):
        return iter(self.commands.items())

    def add_command(self, name, command_class):
        if name == 'help':
            # Overwrite HelpCommand with one which supports commands in the
            # <command> <sub command> class
            command_class = HelpCommand

        self.commands[name]['index'] = EntryPointWrapper(name, command_class)

    def find_command(self, argv, called_by_help=False):
        command = argv[0]

        if command == 'help':
            command_entry = self.commands.get('help', {}).get('index', None)
            cmd_factory = command_entry.load()
            args = argv[1:]
            return (cmd_factory, command, args)

        if len(argv) > 1:
            start_index = 2
            sub_command = argv[1]
        else:
            start_index = 1
            sub_command = 'index'

        command_entry = self.commands.get(command, {}).get(sub_command, None)

        if not command_entry:
            if called_by_help:
                raise ValueError('Unknown command: %r' %
                                 (' '.join(argv),))
            else:
                command_entry = self.commands.get('help', {}) \
                                             .get('index', None)
                cmd_factory = command_entry.load()
                args = [command]
                return (cmd_factory, command, args)

        cmd_factory = command_entry.load()
        args = argv[start_index:]
        command_name = '%s %s' % (command, sub_command)
        return (cmd_factory, command_name, args)



# Copyright 2012 Rackspace
#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import sys

from cliff.command import Command


class HelpAction(argparse.Action):
    """
    Custom HelpAction which recognizes commands in <command> <sub command>
    format.
    """
    def __call__(self, parser, namespace, values, option_string=None):
        app = self.default
        parser.print_help(app.stdout)
        app.stdout.write('\nCommands:\n')
        command_manager = app.command_manager

        for command, sub_commands in sorted(command_manager):
            for sub_command, ep in sub_commands.items():

                try:
                    factory = ep.load()
                except Exception as err:
                    app.stdout.write('Could not load %r\n' % ep)
                    continue

                try:
                    cmd = factory(self, None)
                except Exception as err:
                    app.stdout.write('Could not instantiate %r: %s\n' %
                                    (ep, err))
                    continue
                one_liner = cmd.get_description().split('\n')[0]

                if sub_command == 'index':
                    name = command
                else:
                    name = '%s %s' % (command, sub_command)

                app.stdout.write('  %-13s  %s\n' % (name, one_liner))
        sys.exit(0)


class HelpCommand(Command):
    """print detailed help for another command
    """

    def get_parser(self, prog_name):
        parser = super(HelpCommand, self).get_parser(prog_name)
        parser.add_argument('cmd',
                            nargs='*',
                            help='name of the command',
                            )
        return parser

    def take_action(self, parsed_args):
        if parsed_args.cmd:
            try:
                the_cmd = self.app.command_manager.find_command(
                    parsed_args.cmd, called_by_help=True)
                cmd_factory, cmd_name, search_args = the_cmd
            except ValueError:
                # Did not find an exact match
                cmd = parsed_args.cmd[0]
                fuzzy_matches = [k for k in self.app.command_manager
                                 if k[0].startswith(cmd)
                                 ]
                if not fuzzy_matches:
                    raise
                self.app.stdout.write('Command "%s" matches:\n\n' % (cmd))
                for fm in fuzzy_matches:
                    for sub_command, _ in fm[1].items():
                        self.app.stdout.write(' - %s %s\n' % (fm[0],
                                                              sub_command))
                return
            cmd = cmd_factory(self.app, search_args)
            full_name = (cmd_name
                         if self.app.interactive_mode
                         else ' '.join([self.app.NAME, cmd_name])
                         )
            cmd_parser = cmd.get_parser(full_name)
        else:
            cmd_parser = self.get_parser(' '.join([self.app.NAME, 'help']))
        cmd_parser.print_help(self.app.stdout)
        return 0



# Copyright 2012 Rackspace
#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from cliff.formatters.table import TableFormatter

__all__ = [
    'PaginatedListFormatter'
]


class PaginatedListFormatter(TableFormatter):
    def add_argument_group(self, parser):
        pass

    def emit_list(self, column_names, data, stdout, parsed_args):
        super(PaginatedListFormatter, self).emit_list(column_names, data,
                                                      stdout, parsed_args)

        returned_metadata = getattr(parsed_args, 'returned_metadata', {})
        limit = returned_metadata.get('limit', None)
        marker = returned_metadata.get('marker', None)
        next_marker = returned_metadata.get('next_marker', None)

        if len(data) >= 1 and limit:
            print ''
            print 'Displaying: %s' % (len(data))

        if limit:
            print 'Limit: %s' % (limit)

        if marker:
            print 'Marker in use: %s' % (marker)

        if next_marker:
            print 'Next marker: %s' % (next_marker)



#!/usr/bin/env python
# Copyright 2012 Rackspace
#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os
import sys

from distutils.util import convert_path
from distutils.core import Command
from fnmatch import fnmatchcase
from setuptools import setup, find_packages
from subprocess import call


def read_version_string():
    version = None
    sys.path.insert(0, os.path.join(os.getcwd()))
    from cliff_rackspace import __version__
    version = __version__
    sys.path.pop(0)
    return version


setup(
    name='cliff-rackspace',
    version=read_version_string(),
    description='Rackspace Python cliff library extensions.',
    author='Rackspace Hosting',
    author_email='sr@rackspace.com',
    url='https://github.com/racker/python-cliff-rackspace',
    classifiers=['Development Status :: 4 - Beta',
                 'License :: OSI Approved :: Apache Software License',
                 'Programming Language :: Python',
                 'Programming Language :: Python :: 2.6',
                 'Programming Language :: Python :: 2.7',
                 'Programming Language :: Python :: Implementation :: PyPy',
                 'Environment :: Console',
                 ],
    platforms=['Any'],
    scripts=[],
    provides=[],
    install_requires=[],
    namespace_packages=[],
    packages=find_packages()
)
